@@ -36,15 +36,17 @@ class SparqlProvider(WikidataProvider):
         self.cache = {}
 
     def query_wikidata_for_property_type(self, wikidata_property):
-        query = """SELECT ?type ?label ?desc WHERE {
-                    wd:"""+wikidata_property+""" rdf:type wikibase:Property;
-                        wikibase:propertyType ?type.
-                    OPTIONAL { wd:"""+wikidata_property+""" rdfs:label ?label. }
-                    OPTIONAL { wd:"""+wikidata_property+""" schema:description ?desc. }
-                    FILTER(LANGMATCHES(LANG(?label), "EN"))
-                    FILTER(LANGMATCHES(LANG(?desc), "EN"))
-                    }"""
-        sparql = SPARQLWrapper(self.sparql_endpoint)
+        query = """SELECT ?label ?desc ?type
+                WHERE 
+                {{
+                wd:{wpid} wikibase:propertyType ?type;
+                SERVICE wikibase:label {{ bd:serviceParam wikibase:language "en".
+                        wd:{wpid} rdfs:label         ?label.
+                        wd:{wpid} schema:description   ?desc.
+                    }}
+                }}
+                """.format(wpid=wikidata_property)
+        sparql = SPARQLWrapper(self.sparql_endpoint, agent='Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/50.0.2661.102 Safari/537.36')
         sparql.setQuery(query)
         sparql.setReturnFormat(JSON)
         results = sparql.query().convert()
@@ -61,13 +63,12 @@ class SparqlProvider(WikidataProvider):
     def get_property_type(self, wikidata_property: str):
         data_type = self.cache.get(wikidata_property, False)
         if not data_type:
-            data_type_args = self.query_wikidata_for_property_type(
+            property_args = self.query_wikidata_for_property_type(
                 wikidata_property)
-            data_type=data_type_args["data_type"]
-            self.save_entry(wikidata_property, **data_type_args)
+            data_type=property_args["data_type"]
+            self.save_entry(wikidata_property, **property_args)
             if data_type == "Property Not Found":
                 raise ValueError("Property "+wikidata_property+" not found")
-
         return data_type
 
     def save_entry(self, property, data_type, *args, **kwargs):
